"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _isobject = _interopRequireDefault(require("isobject"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * The Store class. It's a key value store with simple subscription support.
 */
var Store =
/*#__PURE__*/
function () {
  /**
   * Constructor
   */
  function Store(data) {
    var _this = this;

    _classCallCheck(this, Store);

    _defineProperty(this, "isAlfaStore", true);

    _defineProperty(this, "silentVersion", 0);

    _defineProperty(this, "setWithOutputs", function (outputs) {
      var set = _this.set;
      return function checkOutputAndSet(key, value, maps) {
        if ((0, _isobject.default)(key)) {
          Object.keys(key).forEach(function (_key) {
            checkOutputAndSet(_key, key[_key], maps);
          });
          return;
        }

        if (Array.isArray(outputs) && outputs.indexOf(key) === -1) {
          // Throw exception if the output key is not allowed.
          throw new Error("Output key \"".concat(key, "\" is not allowed. You need to define it as an output when calling inject/subscribe."));
        }

        if (maps && maps[key]) {
          key = maps[key];
        }

        set(key, value);
      };
    });

    _defineProperty(this, "clone", function () {
      var cloned = {};
      var _store = _this._store;
      Object.keys(_store).forEach(function (key) {
        return cloned[key] = _store[key];
      });
      return cloned;
    });

    _defineProperty(this, "subscribe", function (keys, fn, maps) {
      if ('function' !== typeof fn) {
        throw new TypeError('`fn` must be a function');
      }

      if (maps) {
        fn.maps = maps;
      }

      var _subscriptions = _this._subscriptions;
      Array.isArray(keys) && keys.forEach(function (key) {
        var subs = _subscriptions[key];

        if (Array.isArray(subs)) {
          subs.indexOf(key) === -1 && subs.push(fn);
        } else {
          _subscriptions[key] = [fn];
          return;
        }
      });
    });

    _defineProperty(this, "unsubscribe", function (fn) {
      var _subscriptions = _this._subscriptions;
      Object.keys(_subscriptions).forEach(function (key) {
        var subs = _subscriptions[key];
        subs && (_subscriptions[key] = subs.filter(function (f) {
          return f !== fn;
        }));
      });
    });

    /**
     * Internal object which holds the key/value map.
     * @type {Object}
     */
    this._store = {};
    /**
     * Internal object which holds all the subscription functions.
     * @type {Object}
     */

    this._subscriptions = {};
    this.set = this.set.bind(this); // Initial store if we have data.

    data && this.set(data);
  }
  /**
   * @type {Boolean}
   */


  _createClass(Store, [{
    key: "get",

    /**
     * Get value from store by key.
     *
     * @param  {String|undefined} key Name of the value to get.
     * @return {Any}        Value.
     */
    value: function get(key) {
      var keyType = _typeof(key);

      var _store = this._store;

      if ('string' === keyType) {
        return _store[key];
      } else if (Array.isArray(key) && key.length > 0) {
        var results = {};
        key.forEach(function (k) {
          if ('string' === typeof k) {
            if (_store.hasOwnProperty(k)) {
              results[k] = _store[k];
            }
          } else {
            throw new TypeError('Type of `key` must be string, array of strings or undefined.');
          }
        });
        return results;
      } else if ('undefined' === keyType) {
        return this.clone();
      } else {
        throw new TypeError('Type of `key` must be string, array of strings or undefined.');
      }
    }
    /**
     * Save the `value` in store with name `key`.
     *
     * @param {String|Object}   key   Name of the value in store.  Or object of
     * key/value pairs to merge into the store.
     * @param {Any}             value Value to save.
     * @param {Boolean}         silent set to true to avoid calling subscription
     *                          functions.
     */

  }, {
    key: "set",
    value: function set(key, value, silent) {
      var store = this;

      if ('string' === typeof key) {
        setSingle(store, key, value, silent);
      } else if ((0, _isobject.default)(key)) {
        Object.keys(key).forEach(function (_key) {
          setSingle(store, _key, key[_key], silent);
        });
      } else {
        throw new TypeError('Type of `key` must be string or plain object.');
      }
    }
  }]);

  return Store;
}();
/**
 * Set a single value to an object.
 */


exports.default = Store;

function setSingle(store, key, value, silent) {
  var _store = store._store,
      _subscriptions = store._subscriptions; // Uncurry alfa action functions

  if (typeof value === 'function' && value.alfaAction) {
    value = value.alfaAction(store);
  } // Save the value to the store.


  _store[key] = value;

  if (silent) {
    store.silentVersion += 1;
    return;
  } // Call subscribed functions if we have.


  var subs = _subscriptions[key];

  if (subs) {
    var changed = _defineProperty({}, key, value);

    subs.forEach(function (subFn) {
      // Make sure the subFn is still legit at the time we are calling it since
      // all subscribing functions are actually `setStat`.  And previous
      // `setState` calls could trigger unmount component which later `setState`
      // belongs to.
      // Need to make the below code reproducible by test.
      // var _subs = subscriptions[key]
      // if (_subs.indexOf(subFn) === -1) {
      //   return
      // }
      if (subFn.maps) {
        var maps = subFn.maps;
        Object.keys(maps).some(function (injectKey) {
          var realKey = maps[injectKey];

          if (realKey === key) {
            changed = _defineProperty({}, injectKey, value);
            return true;
          }

          return false;
        });
      }

      subFn(changed);
    });
  }
}