"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.normalizeInputs = normalizeInputs;
exports.normalizeOutputs = normalizeOutputs;
exports.subscribe = exports.inject = void 0;

var _isobject = _interopRequireDefault(require("isobject"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _react = require("react");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

/**
 * Public API
 */
var inject = createInjector('inject', createAlfaInjectedComponent);
exports.inject = inject;
var subscribe = createInjector('subscribe', createAlfaSubscribedComponent);
/**
 * Private functions
 */

exports.subscribe = subscribe;

function createInjector(type, creator) {
  return function (WrappedComponent, inputs, outputs) {
    if (typeof WrappedComponent === 'function') {
      var componentName = WrappedComponent.name;
      inputs = normalizeInputs(componentName, inputs, WrappedComponent.keys);
      outputs = normalizeOutputs(componentName, inputs, outputs);
      return creator(WrappedComponent, inputs, outputs);
    } else {
      // istanbul ignore next
      var error = "alfa.".concat(type, " only accepts function or class. Got \"").concat(_typeof(WrappedComponent), "\".");
      throw new TypeError(error);
    }
  };
}

function normalizeInputs(name, inputs, dynamicInputs) {
  if (inputs && 'string' === typeof inputs) {
    return [inputs];
  } else if (Array.isArray(inputs)) {
    return inputs;
  } else if ('function' === typeof dynamicInputs) {
    return [];
  } else {
    throw new TypeError("".concat(name, ": inject/subscribe only accepts string or array of strings as second parameter (inputs) when static property 'keys' of component does not exist."));
  }
}

function normalizeOutputs(name, inputs, outputs) {
  // Check if output keys are provided when `set` is required as input key.
  if (Array.isArray(inputs) && inputs.indexOf('set') > -1 && (!Array.isArray(outputs) || 0 === outputs.length)) {
    throw new Error("".concat(name, ": outputs are required as 3rd argument of function \"inject/subscribe\" when \"set\" is injected/subscribed."));
  }

  if (outputs) {
    // When we have key(s) of output we need to check the type(s) of all the keys.
    if ('string' === typeof outputs) {
      // The outputs is a string then normalize it as an array.
      return [outputs];
    }

    if (Array.isArray(outputs) && outputs.every(function (key) {
      return typeof key === 'string';
    })) {
      // Outputs is an array, make sure all the elements of this array are string.
      return outputs;
    } // Throw exception if any key of the outputs is not supported.


    throw new TypeError("".concat(name, ": inject/subscribe only accepts string or array of strings as 3rd parameter (outputs)."));
  }
}

function createAlfaInjectedComponent(WrappedComponent, inputs, outputs) {
  var keys = WrappedComponent.keys;

  function AlfaInjectedComponent(props, context, updater) {
    var alfaStore = context && context.alfaStore;
    var injectedProps = getInjectedProps(inputs, outputs, alfaStore); // Props passed in directly to constructor has lower priority than inputs
    // injected from the store.

    var _props = _objectSpread({}, props, injectedProps);

    if (keys) {
      var dynamicInputs = getDynamicInputs(keys, _props, inputs);
      _props = getDynamicProps(dynamicInputs, outputs, alfaStore);
    }

    if (WrappedComponent.prototype.isReactComponent) {
      // Create an element if it's react component.
      return (0, _react.createElement)(WrappedComponent, _props);
    } else {
      // Otherwise, call the original function.
      return WrappedComponent(_props, context, updater);
    }
  }

  AlfaInjectedComponent.keys = keys;
  AlfaInjectedComponent.contextTypes = {
    alfaStore: _propTypes.default.object
  };
  return AlfaInjectedComponent;
}

function createAlfaSubscribedComponent(WrappedComponent, inputs, outputs) {
  var keys = WrappedComponent.keys;

  var AlfaSubscribedComponent =
  /*#__PURE__*/
  function (_Component) {
    _inherits(AlfaSubscribedComponent, _Component);

    function AlfaSubscribedComponent(_props2, context, updater) {
      var _this;

      _classCallCheck(this, AlfaSubscribedComponent);

      // Call the original constructor.
      _this = _possibleConstructorReturn(this, _getPrototypeOf(AlfaSubscribedComponent).call(this, _props2, context, updater));
      /* istanbul ignore next */
      // Save the store for subscribe/unsubscribe.

      _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "getState", function (props, alfaStore) {
        if (!alfaStore || _this.silentVersion === alfaStore.silentVersion) {
          return _this.state;
        } // Sync silent version with alfa store


        _this.silentVersion = alfaStore.silentVersion; // Get injected props which is part of the state of the component.

        var injectedProps = getInjectedProps(inputs, outputs, alfaStore); // Merge injected props with props where the injectedProps has higher priority.

        var _props = _objectSpread({}, props, injectedProps); // Get dynamic inputs.


        var dynamicInputs = getDynamicInputs(keys, _props, inputs);
        _this.maps = dynamicInputs.maps;
        _this.inputs = dynamicInputs.inputs; // Get the props and return as state

        return getDynamicProps(dynamicInputs, outputs, alfaStore);
      });

      _this.store = context && context.alfaStore; // Get the state.
      // console.log('silentVersion in constructor', this.silentVersion)

      _this.state = _this.getState(_props2, _this.store); // Use bound setState as subscription function.

      var setState = _this.setState.bind(_assertThisInitialized(_assertThisInitialized(_this)));

      _this.setState = function () {
        setState.apply(null, arguments);
      };

      return _this;
    }

    _createClass(AlfaSubscribedComponent, [{
      key: "componentDidMount",
      value: function componentDidMount() {
        // Call `setState` when subscribed keys changed.
        this.store && this.store.subscribe(this.inputs, this.setState, this.maps);
      }
    }, {
      key: "componentWillUnmount",
      value: function componentWillUnmount() {
        // Unsubcribe `setState` when component is about to unmount.
        this.store && this.store.unsubscribe(this.setState);
      }
    }, {
      key: "render",
      value: function render() {
        // Render the original component with state as its props.
        return (0, _react.createElement)(WrappedComponent, _objectSpread({}, this.props, this.getState(this.props, this.store)));
      }
    }]);

    return AlfaSubscribedComponent;
  }(_react.Component);

  _defineProperty(AlfaSubscribedComponent, "contextTypes", {
    alfaStore: _propTypes.default.object
  });

  AlfaSubscribedComponent.keys = keys;
  return AlfaSubscribedComponent;
}

function getInjectedProps(inputs, outputs, alfaStore) {
  var stringInputs = inputs.filter(function (input) {
    return typeof input === 'string';
  });

  var injectedProps = _objectSpread({}, alfaStore.get(stringInputs)); // Need to inject set.


  if (inputs.indexOf('set') > -1) {
    injectedProps.set = alfaStore.setWithOutputs(outputs);
  }

  return injectedProps;
}
/**
 * Load dependencies with the result of calling `keys` function of the component.
 *
 * This gives people the ability to load dynamic dependencies based on the props
 * of the component at runtime.
 * It makes a map between the dynamic names of the dependencies and the names
 * of the properties injected in `state` of the component.
 * That helps maintaining a simple naming system in the application code.
 *
 * @param  {Function} keys
 * @param  {Object} props
 * @param  {Array} outputs
 * @param  {Object} alfaStore
 * @return {Object}
 */


function getDynamicProps(_ref, outputs, alfaStore) {
  var keys = _ref.keys,
      maps = _ref.maps,
      inputs = _ref.inputs;

  var _props = getInjectedProps(inputs, outputs, alfaStore);

  if (maps) {
    keys.forEach(function (key) {
      _props[key] = _props[maps[key]];
    }); // Map outputs

    if (outputs && 'function' === typeof _props.set) {
      // The `set` of `props` which is obtained from calling
      // `alfaStore.setWithOutputs(outputs)`
      var _setWithOutputs = _props.set; // Call `_setWithOutputs` with maps if

      _props.set = function (key, value) {
        _setWithOutputs(key, value, maps);
      };
    }

    return _props;
  } else {
    // No mapped props
    return _props;
  }
}

function getDynamicInputs(getKeys, props, _inputs) {
  if (getKeys && 'function' === typeof getKeys) {
    var inputs = getKeys(props);

    if (Array.isArray(inputs)) {
      return {
        inputs: _toConsumableArray(_inputs).concat(_toConsumableArray(inputs))
      };
    } else if ((0, _isobject.default)(inputs)) {
      // Object of mappings between injection keys and real input keys.
      var keys = Object.keys(inputs);
      var realInputs = keys.map(function (key) {
        return inputs[key];
      });
      return {
        keys: keys,
        maps: inputs,
        inputs: _toConsumableArray(_inputs).concat(_toConsumableArray(realInputs))
      };
    }
  }

  return {
    inputs: _inputs
  };
}